//
//  NeoFinderAIBundleMain
//
//  Created by norbert doerner on 21.08.20
//  Copyright © 2020 norbert doerner. All rights reserved.
//

#import <CoreVideo/CoreVideo.h>
#import "NeoFinderAIBundleMain.h"
#import <Vision/Vision.h>


@interface nmacOSAIBundleMain ()
{
	NSArray		*mAllKeywords;
}
@end

@implementation nmacOSAIBundleMain

- (void)prepareTagList
{
	if (mAllKeywords != NULL)
		return;
	
	NSError			*myErr = NULL;
	NSBundle		*myBundle = [NSBundle bundleForClass:[self class]];	// mainBundle would be the Host app NeoFinder, which is not what we need
	NSString		*fileListPath = [myBundle pathForResource:@"Wordlist" ofType:@"txt"];
	if (fileListPath == NULL)
		return;
	NSString		*keywordsFileContent = [NSString stringWithContentsOfFile:fileListPath encoding:NSUTF8StringEncoding error:&myErr];
	mAllKeywords = [keywordsFileContent componentsSeparatedByString:@"\n"];
}  //  prepareTagList

- (int)setupEngine
{
//	NSLog (@"NeoFinderAIBundleMain setupEngine");
	return 0;
}

- (int)pluginVersion
{
	return kCurrentPluginVersion;
}

- (NSString *_Nonnull)engineNameForDisplay
{
	return @"macOS AI Engine 1.1 by NeoFinder";
}


// can this bundle run in this hardware and software?
- (BOOL)canRun
{
	if (@available(macOS 10.15, *)) 
	{
		return YES;
	}
	return NO;
}  //  canRun

- (NSArray *)listOfSupportedTags	// a list of NSStrings: all the Tags that this engine can recognize. Localized! Can be used for GUI things
{
	return mAllKeywords;
}


- (void)prepareDetectionEngine
{
	// nothing to do here
}

- (NSSize)sizeNeededForPixBuffers
{
	return NSMakeSize(224, 224);
}

// return all tags and their probabilities. Keys in the resulting Dict are the tag names
- (NSDictionary *)tagsForImage:(CVPixelBufferRef)theImagePixBuffer error:(NSError * _Nullable * _Nullable)error;
{
	if (@available(macOS 10.15, *)) 
	{
		__block NSMutableDictionary		*result = [NSMutableDictionary dictionary];
		NSDictionary			*options = [NSDictionary dictionary];
		VNImageRequestHandler		*myHandler = [[VNImageRequestHandler alloc] initWithCVPixelBuffer:theImagePixBuffer options:options];
		VNClassifyImageRequest		*myRequest = [[VNClassifyImageRequest alloc] initWithCompletionHandler:^(VNRequest * _Nonnull request, NSError * _Nullable error) 
		{
			for (VNClassificationObservation *oneResult in request.results)
			{
				BOOL	myVal = [oneResult hasMinimumPrecision:0.5 forRecall:0.5];
				if (myVal)
					[result setObject:[NSNumber numberWithFloat:[oneResult confidence]] forKey:[oneResult identifier]];
			}
		}];
		NSError		*error = NULL;
		BOOL		didWork = [myHandler performRequests:[NSArray arrayWithObject:myRequest] error:&error];
		if (didWork == YES)
		{
			if ([result count] > 0)	// we have found something
				return result;
		}
	}
	return NULL;
}  //  tagsForImage

- (void)shutDownDetectorEngine
{
	mAllKeywords = NULL;
}

@end
